{**
	CESYS Software License - Version 1.0 - January 1st, 2016

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The software must only be used to operate hardware manufactured by 
	CESYS GmbH, Herzogenaurach.

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
  
	API Interface for communicating with CeboMsr-Devices.

	Copyright (c) 2016 Cesys GmbH

	@author Daniel Schroll
}
unit CeboMsr;

interface
	uses SysUtils;
	type

		// Exceptions
		EArgumentException = Class(Exception);
		EIndexOutOfRangeException = Class(Exception);
		EInvalidOperationException = Class(Exception);
		EIOException = Class(Exception);
		EApplicationException = Class(Exception);
		ENotImplemented = Class(Exception);

		//
		//// Types
		//

		CeboMsrType = longword;
		CeboMsrErrorIndex = longword;
		CeboMsrHandle = longword;
		CeboMsrIo = longword;
		CeboMsrRange = longword;
		CeboMsrCurrentSource = longword;
		CeboMsrLed = longword;
		CeboMsrTriggerConfigId = longword;
		CeboMsrCounterConfigId = longword;
		DigitalArray = array of longint;
		VoltageArray = array of single;
		IoArray = array of CeboMsrIo;
		TStringArray = array of ansistring;

		// interface for transfering min and max values of TRange
		TRangeMinMax = array [1..2] of single;

		// enumeration for TAnalogInput
		TStyles = (SingleEnded, Differential);

		// Stringtype for pansichar to Ansistring conversion in SystemConnector
		TStringType = array [1..1024] of char;

		//
		//// Class Definitions
		//
		
		{** Direct access to the CeboMsr-Base-API. Internal use only.}
		TSystemConnector = class
			private
				class procedure FillSArray(var arr: TStringType);
				class procedure CheckError(ErrorCode: CeboMsrErrorIndex);
				class function GetApiVersion(): ansistring;
				class function BeginEnumerate(MsrType: CeboMsrType): longint;
				class function BeginEnumerateNetwork(Hostname: ansistring; Port: longint; MsrType: CeboMsrType): longint;
				class function GetIdentifier(Id: longint; var MsrType: CeboMsrType; var Version: ansistring): ansistring;
				class function Open(Ident: ansistring):CeboMsrHandle;
				class procedure Close(Handle: CeboMsrHandle);
				class procedure ResetDevice(Handle: CeboMsrHandle);
				class procedure ResetPeripherals(Handle: CeboMsrHandle; Mask: longint);
				class function GetDeviceParameter(Handle: CeboMsrHandle; Parameter: longint): longint;
				class function GetDigitalPortIoCount(Handle: CeboMsrHandle; IoId: CeboMsrIo): longint;
				class function GetPeripheralName(Handle: CeboMsrHandle; IoId: CeboMsrIo): ansistring;
				class function GetDigitalPortIoName(Handle: CeboMsrHandle; IoId: CeboMsrIo; Io: longint): ansistring;
				class function EnumerateRanges(Handle: CeboMsrHandle; IoId: CeboMsrIo; Id: longint): CeboMsrRange;
				class function GetRangeMinMax(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange):TRangeMinMax;
				class function GetDefaultAnalogInputInterChannelDelay(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange):longint;
				class function GetMinAnalogInputInterChannelDelay(Handle: CeboMsrHandle; IoId: CeboMsrIo):longint;
				class function GetSerialNumber(Handle: CeboMsrHandle): ansistring;
				class function GetTemperature(Handle: CeboMsrHandle): single;
				class procedure SetLed(Handle: CeboMsrHandle; IoId: CeboMsrIo; State: boolean);
				class function GetReferenceCurrent(Handle: CeboMsrHandle; IoId: CeboMsrIo): single;
				class procedure SetDigitalPortOutputEnableMask(Handle: CeboMsrHandle; IoId: CeboMsrIo; Mask: longint);
				class function SetAnalogInputParameters(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange; InterChannelDelay: longint): longint;
				class function GetAnalogInputParameters(Handle: CeboMsrHandle; IoId: CeboMsrIo; InterChannelDelay: longint): CeboMsrRange;
				class procedure SetTriggerEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; State: boolean);
				class function GetTriggerEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo): boolean;
				class procedure SetTriggerConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; TriggerConfigId: CeboMsrTriggerConfigId);
				class function GetTriggerConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo): CeboMsrTriggerConfigId;
				class procedure ResetCounter(Handle: CeboMsrHandle; IoId: CeboMsrIo);
				class procedure SetCounterEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; State: boolean);
				class function GetCounterEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo): boolean;
				class procedure SetCounterConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; CounterConfigId: CeboMsrCounterConfigId);
				class function GetCounterConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo): CeboMsrCounterConfigId;
				class function CalculateMaxBufferedInputFrames(Handle: CeboMsrHandle): longint;
				class procedure SetupInputFrame(Handle: CeboMsrHandle; var Ios: IoArray);
				class function StartDataAcquisition(Handle: CeboMsrHandle; ExternalStarted: boolean; FrameRate: single; FrameCount: longint): single;
				class procedure StopDataAcquisition(Handle: CeboMsrHandle);
				class procedure ReadBlocking(Handle: CeboMsrHandle; var VBuffer: VoltageArray; var Buffer: DigitalArray; FrameCount: longint);
				class function ReadNonBlocking(Handle: CeboMsrHandle; VBuffer: VoltageArray; Buffer: DigitalArray): longint;
				class function ReadDigitalPort(Handle: CeboMsrHandle; IoId: CeboMsrIo): longint;
				class function ReadAnalogInput(Handle: CeboMsrHandle; IoId: CeboMsrIo): single;
				class procedure ReadFrame(Handle: CeboMsrHandle; VBuffer: VoltageArray; Buffer: DigitalArray);
				class function ReadCounter(Handle: CeboMsrHandle; IoId: CeboMsrIo): int64;
				class procedure SetupOutputFrame(Handle: CeboMsrHandle; Ios: IoArray);
				class procedure WriteDigitalPort(Handle: CeboMsrHandle; IoId: CeboMsrIo; MsrValue: longint);
				class procedure WriteAnalogOutput(Handle: CeboMsrHandle; IoId: CeboMsrIo; MsrValue: single);
				class procedure WriteFrame(Handle: CeboMsrHandle; VBuffer: VoltageArray; Buffer: DigitalArray);
		end;

		{**
			These objects are used to set the range of analog inputs and outputs.
			Direct construction is not possible, SupportedRanges of AnalogInput and
			AnalogOutput can be used to request the possible ranges of an input or output.
		}
		TRange = class
			protected
				_RangeId: CeboMsrRange;
				_MinValue, _MaxValue: single;
				_DefaultInterChannelDelay: longint;
			private
				property RangeId: CeboMsrRange read _RangeId;
				property DefaultInterChannelDelay: longint read _DefaultInterChannelDelay;
				constructor Create(RngeId: CeboMsrRange; MinVal, MaxVal: single; DefInterChannelDelay: longint);
			public
				property MinValue: single read _MinValue; //** Lower Voltage.
				property MaxValue: single read _MaxValue; //** Upper Voltage.
				function ToString(): string; override;
		end;
		TRangeList = array of TRange;

		{** Interface for any object that can act as input.}
		IInput = interface
		end;
		TIInputList = array of IInput;

		{** Forward declaration of Device-Class }
		TDevice = class;

		{**
			Offers functionality of the different kind of analog inputs.
			Direct construction is not possible, instances can be retrieved from
			SingleEndedInputs or DifferentialInputs of the respective instance
			of class Device.
		}
		TAnalogInput = class(TInterfacedObject, IInput)
			protected
				_SupportedRanges: TRangeList;
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Style: TStyles;
				_Index: longint;
				_MinInterChannelDelay: longint;
				_Name: ansistring;
				function GetRange(): TRange;
				function GetInterChannelDelay(): longint;
				function GetVoltageValue(): single;
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Style: TStyles read _Style;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Styl: TStyles; Idx: longint);
			public
				{** Request default interchannel delay at the given range setting.
					@param(ARange Range for which this value is requested.)
				}
				function GetDefaultInterChannelDelay(ARange: TRange): longint;
				{** Set new range, interchannel delay is set to the default for this range.
					@param(ARange Range level to set.)
				}
				procedure SetParameters(ARange: TRange); overload;
				{** Set range and user-specified interchannel delay.
					@param(ARange Range level to set.)
					@param(InterChannelDelay Interchannel delay in microseconds.)
					@returns(Corrected interchannel delay in microseconds.)
				}
				function SetParameters(ARange: TRange;InterChannelDelay: longint):longint; overload;
				function ToString(): string; override;
				{** List of the supported ranges. }
				property SupportedRanges: TRangeList read _SupportedRanges;
				{** Minimum interchannel delay. }
				property MinInterChannelDelay: longint read _MinInterChannelDelay;
				{** Define (identical to SetParameters(ARange)) or read active range setting.}
				property Range: TRange read GetRange write SetParameters;
				{** Active interchannel delay.}
				property InterChannelDelay: longint read GetInterChannelDelay;
				{** Read current voltage value.}
				property Value: single read GetVoltageValue;
				{** Name of the component.}
				property Name: ansistring read _Name;
		end;
		TAnalogInputList = array of TAnalogInput;

		{** Interface for any object that can act as output. }
		IOutput = interface
		end;
		TIOutputList = array of IOutput;

		{**
			Offers functionality for analog outputs. Direct construction
			is not possible. Instances can be retrieved from AnalogOutputs
			of the respective Device class.
		}
		TAnalogOutput = class(TInterfacedObject, IOutput)
			protected
				_SupportedRanges: TRangeList;
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Index: longint;
				_Name: ansistring;
				function GetRange(): TRange;
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
				procedure WriteValue(Value: single);
			public
				{** Set new range.
					@parameter(Range)
				}
				procedure SetParameters(Range: TRange);
				{** List of the supported ranges.}
				property SupportedRanges: TRangeList read _SupportedRanges;
				{** Define (identical to SetParameters(Range)) or read active range setting.}
				property Range: TRange read GetRange write SetParameters;
				{** Set Voltage level directly.}
				property Value: single write WriteValue;
				{** Name of the component.}
				property Name: ansistring read _Name;
				function ToString(): string; override;
		end;
		TAnalogOutputList = array of TAnalogOutput;

		{**
			Offers functionality to work with digital ports. Direct construction
			is not possible. Instances can be retrieved from DigitalPorts of
			the respective Device instance.
		}
		TDigitalPort = class(TInterfacedObject, IInput, IOutput)
			protected
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Index: longint;
				_Name: ansistring;
				_IoCount: longint;
				_IoNames: TStringArray;
				procedure SetOutputEnableMask(Mask: longint);
				function GetValue(): longint;
				procedure SetValue(val: longint);
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
			public
				{** Set output mask. The default mask is 0. Mask to use, bits that have state '1' specify output direction.}
				property OutputEnableMask: longint write SetOutputEnableMask;
				{** Count of I/O's for the specified port.}
				property IoCount: longint read _IoCount;
				{**
					Set output I/O's or read them. Only the bits that have been defined as output
					using OutputEnableMask are modified during write.
				}
				property Value: longint read GetValue write SetValue;
				{** Name of the component.}
				property Name: ansistring read _Name;
				{** Return name othe I/O index in range ( 0 <= Io < GetIoCount()).}
				function GetIoName(Io: longint): ansistring;
				function ToString(): string; override;
		end;
		TDigitalPortList = array of TDigitalPort;

		{** The possible counter configuration values. }
		TCounterConfig = (
			RisingEdge = $ceb01a00, //Constant: CeboMsrCounterConfigRisingEdge
			FallingEdge = $ceb01a01, //Constant: CeboMsrCounterConfigFallingEdge
			Alternating = $ceb01a02 //Constant: CeboMsrCounterConfigAlternating
		);

		{**
			Offers functionality for counter periphery. Instances can be retrieved from
			Counters of the respective Device instance.
		}
		TCounter = class(TInterfacedObject, IInput)
			protected
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Index: longint;
				_Name: ansistring;
				procedure SetEnabled(State: boolean);
				function GetEnabled(): boolean;
				procedure SetConfig(Conf: TCounterConfig);
				function GetConfig(): TCounterConfig;
				function GetValue(): Int64;
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
			public
				{** Reset counter to 0.}
				procedure Reset();
				{** Define or retrieve the counters enabled state.}
				property Enabled: boolean read GetEnabled write SetEnabled;
				{** Define and request behavior of the counter.}
				property Config: TCounterConfig read GetConfig write SetConfig;
				{** Current counter value.}
				property Value: Int64 read GetValue;
				{** Name of the component.}
				property Name: ansistring read _Name;
				function ToString(): string; override;
		end;
		TCounterList = array of TCounter;

		{** The possible trigger configuration values.}
		TTriggerConfig = (
			OutputPulse = $ceb01900, //Constant: CeboMsrTriggerOutputPulse
			OutputAlternating = $ceb01901, //Constant: CeboMsrTriggerOutputAlternating
            InputRisingEdge = $ceb01902, //Constant: CeboMsrTriggerInputRisingEdge
            InputFallingEdge = $ceb01903, //Constant: CeboMsrTriggerInputFallingEdge
            InputAlternating = $ceb01904 //Constant: CeboMsrTriggerInputAlternating
		);

		TTrigger = class(TInterfacedObject, IInput)
			protected
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Index: longint;
				_Name: ansistring;
				procedure SetEnabled(State: boolean);
				function GetEnabled(): boolean;
				procedure SetConfig(Conf: TTriggerConfig);
				function GetConfig(): TTriggerConfig;
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
			public
				{** Define or retrieve the triggers enabled state.}
				property Enabled: boolean read GetEnabled write SetEnabled;
				{** Define and request behavior of the trigger.}
				property Config: TTriggerConfig read GetConfig write SetConfig;
				{** Name of the compnent.}
				property Name: ansistring read _Name;
				function ToString(): string; override;
		end;
		TTriggerList = array of TTrigger;

		{**
			Offers functionality of for LED's. Instances can be retrieved from Leds
			of the respective Device instance.
		}
		TLed = class
			protected
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Index: longint;
				_Name: ansistring;
				procedure SetEnabled(State: boolean);
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
			public
				{** Set state of the on-board led.}
				property Enabled: boolean write SetEnabled;
				{** Name of the component}
				property Name: ansistring read _Name;
				function ToString(): string; override;
		end;
		TLedList = array of TLed;

		{**
			Offers functionality for current sources. Instances can be retrieved from
			CurrentSources of the respective Device instance.
		}
		TCurrentSource = class
			protected
				_Device: TDevice;
				_InterfaceId: CeboMsrIo;
				_Index: longint;
				_Name: ansistring;
				function GetReference(): single;
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				property Index: longint read _Index;
				constructor Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
			public
				{** The real current of the constant current source in micro ampere.}
				property ReferenceCurrent: single read GetReference;
				{** Name of the component.}
				property Name: ansistring read _Name;
				function ToString(): string; override;
		end;
		TCurrentSourceList = array of TCurrentSource;

		{**
			One or more instances of this interface are returned when reading one or multiple
			frames previously setup using Device.SetupInputFrame(). It allows
			simple access to all captured data.
	   	}
		TInputFrame = class
			protected
				_VValues: VoltageArray;
				_Values: DigitalArray;
				_SingleEndedOffset: longint;
				_SingleEndedCount: longint;
				_DifferentialOffset: longint;
				_DifferentialCount: longint;
				_DigitalPortOffset: longint;
				_DigitalPortCount: longint;
				_CounterOffset: longint;
				_CounterCount: longint;
				_TriggerOffset: longint;
				_TriggerCount: longint;
			private
				constructor Create(VValues: VoltageArray; Values: DigitalArray; SingleEndedOffset, SingleEndedCount, DifferentialOffset, DifferentialCount, DigitalPortOffset, DigitalPortCount, CounterOffset, CounterCount, TriggerOffset, TriggerCount: longint);
			public
				{** Read value from single ended input with given index.
					@param(Index Zero based index of single ended input in question.)
					@returns(Value of this input in voltage.)
				}
				function GetSingleEnded(Index: longint): single;
				{** Read value from differential input with given index.
					@param(Index Zero based index of differential input in question.)
					@returns(Value of this input in voltage.)
				}
				function GetDifferential(Index: longint): single;
				{** Read value from digital port with given index.
					@param(Index Zero based index of digital port in question.)
					@returns(Active mask of digital port.)
				}
				function GetDigitalPort(Index: longint): longint;
				{** Read level from trigger with given index.
					@param(Index Zero based index of trigger in question.)
					@returns(Current level of trigger, @true equals a high level.)
				}
				function GetTrigger(Index: longint): boolean;
				{** Read value from counter with given index.
					@param(Index Zero based index of counter in question.)
					@returns(Current counter value.)
				}
				function GetCounter(Index: longint): longint;
				function ToString(): string; override;
		end;
		TInputFrameList = array of TInputFrame;

		{**
			Class to use when setting multiple outputs simultaneous. Direct construction is prohibited,
			as the count of the the various elements varies from device type to device type. Instances
			must be generated calling Device.CreateOutputFrame().
		}
		TOutputFrame = class
			protected
				_VValues: VoltageArray;
				_Values: DigitalArray;
			private
				property VValues: VoltageArray read _VValues;
				property Values: DigitalArray read _Values;
				constructor Create(VCount: longint; Count: longint);
			public
				{** Define value that should be set on the specified digital port.
					@param(Index Zero based index of port to specify.)
					@param(Value Value to set at this port.)
				}
				procedure SetDigitalPort(Index: longint; Value: longint);
				{** Define value that should be set on the specified analog output.
					@param(Index Zero based index of output to specify.)
					@param(Value Value to set at this output.)
				}
				procedure SetAnalogOutput(Index: longint; Value: single);
				function ToString(): string; override;
		end;

		{** Item to be used in enumeration.}
		TDeviceType = class
			protected
				_InterfaceId: CeboMsrIo;
				_Name: ansistring;
			private
				property InterfaceId: CeboMsrIo read _InterfaceId;
				constructor Create(IfId: CeboMsrIo; Name: ansistring);
			public
				{** Return instance name, "Usb", "AllDevices", ...}
				property Name: ansistring read _Name;
				class function GetDeviceTypeById(Id: CeboMsrType): TDeviceType;
		end;

		{** Type List of all available items for enumeration. Instantiation is: CeboMsr.DeviceType}
		TDeviceTypeList = class
			protected
				_AllDevices : TDeviceType;
				_Usb : TDeviceType;
				_CeboLC : TDeviceType;
				_CeboStick : TDeviceType;
			public
				constructor Create();
				property AllDevices: TDeviceType read _AllDevices;
				property Usb: TDeviceType read _Usb;
				property CeboLC: TDeviceType read _CeboLC;
				property CeboStick: TDeviceType read _CeboStick;
		end;

		{**
			The primary object to communicate with a CeboMsr-compatible device. Every instance holds a single
			connection to a device. Instances are created during enumeration in LibraryInterface.Enumerate().
		}
		TDevice = class
			protected
				_Ident : ansistring;
				_FirmwareVersion : ansistring;
				_MsrType : TDeviceType;
				_Handle : CeboMsrHandle;
				_MaxInputFrames : longint;
				_SingleEnded : TAnalogInputList;
				_Differential : TAnalogInputList;
				_AnalogOutputs : TAnalogOutputList;
				_DigitalPorts : TDigitalPortList;
				_Counters : TCounterList;
				_Triggers : TTriggerList;
				_Leds : TLedList;
				_CurrentSources : TCurrentSourceList;
				function GetInputFrameFloatCount(): longint;
				property InputFrameFloatCount: longint read GetInputFrameFloatCount;
				function GetInputFrameIntCount(): longint;
				property InputFrameIntCount: longint read GetInputFrameIntCount;
				function GetOutputFrameFloatCount(): longint;
				property OutputFrameFloatCount: longint read GetOutputFrameFloatCount;
				function GetOutputFrameIntCount(): longint;
				property OutputFrameIntCount: longint read GetOutputFrameIntCount;
				function BuildInputFrame(VValues: VoltageArray; Values: DigitalArray; FrameNum: longint): TInputFrame;
				
				function GetSerNo(): ansistring;
				function GetTemp(): single;
				
			private
				constructor Create(Ident: ansistring; MsrType: TDeviceType; Version: ansistring);
				property Handle: CeboMsrHandle read _Handle;
			public
				{** Open device.}
				procedure Open();
				{** Close device.}
				procedure Close();
				{** Restore startupdefaults in device.}
				procedure ResetDevice();
				{** Reset spcified elements in device.
					@param(Mask Bit vector constructed of CeboMsr.FlagReset*)
				}
				procedure ResetPeripherals(Mask: longint);
				{** Unique id of device.}
				property Identifier: ansistring read _Ident;
				{** Actual type of device.}
				property DeviceType: TDeviceType read _MsrType;
				{** The firmware version.}
				property FirmwareVersion: ansistring read _FirmwareVersion;
				{** Serial number of device.}
				property SerialNumber: ansistring read GetSerNo;
				{** Read device temperature in degree celsius.}
				property Temperature: single read GetTemp;
				{**
					Calculate maximum count of frames that can be buffered on the device when using
					multiple frame mode. Depends on the actively selected inputs.
					@returns(Frame Count.)
				}
				function CalculateMaxBufferedInputFrames(): longint;
				{**
					Define the inputs that were included in the next frame request
					(ReadFrame(), ReadBlocking() and ReadNonBlocking()).
					@param(Inputs Collection of inputs.)
				}
				procedure SetupInputFrame( Inputs: TIInputList);
				{**
					Initialize a buffered data acquisition. Frames are buffered in the device memory,
					so frame loss is not possible.
					@param(FrameRate Frequency in Hz to capture frame.)
					@param(FrameCount Maximum count of frames to capture. Maximum
						possible frames can be calculated using	CalculateMaxBufferedInputFrames().)
					@param(ExternalStarted If @true, sampling starts if an external
						trigger is detected, immediately otherwise.)
					@returns(Corrected frame rate.)
				}
				function StartBufferedDataAcquisition(FrameRate: single; FrameCount: longint; ExternalStarted: boolean): single;
				{**
					Initialize a buffered data acquisition. Frames are buffered in the device memory,
					so frame loss is not possible. For every detected trigger, one frame is captured.
					Too high frequency on the external trigger will create frame loss.
					@param(FrameCount Maximum count of frames to capture. Maximum
						possible frames can be calculated using CalculateMaxBufferedInputFrames().)
				}
				procedure StartBufferedExternalTimedDataAcquisition(FrameCount: longint);
				{**
					Initialize a continuous data acquisition. Data must be read as fast as possible using
					ReadBlocking()/ReadNonBlocking(), otherwise  an device buffer overflow may occur.
					@param(FrameRate Frequency in Hz to capture frame.)
					@param(ExternalStarted If @true, sampling starts if an external
						trigger is detected, immediately otherwise.)
					@returns(Corrected frame rate.)
				}
				function StartContinousDataAcquisition(FrameRate: single; ExternalStarted: boolean): single;
				{**
					Initialize a continuous data acquisition. Data must be read as fast as possible
					using ReadBlocking()/ReadNonBlocking(), otherwise an device buffer overflow may occur.
					For every detected trigger, one frame is captured. Too high frequency on the external
					trigger will create frame loss.
				}
				procedure StartContinousExternalTimedDataAcquisition();
				{**
					Stop any form of hardware managed data acquisition.
				}
				procedure StopDataAcquisition();
				{**
					Read exact frameCount frames, return after completion, blocks until all data is available.
					@param(FrameCount Count of frames to read.)
					@returns(Collection of read frames.)
				}
				function ReadBlocking(FrameCount: longint): TInputFrameList;
				{**
					Read at much possible available frames, return immediately.
					@returns(Collection of frames available at the moment of
						reading, 0 <= count <= max possible.)
				}
				function ReadNonBlocking(): TInputFrameList;
				{**
					Read multiple inputs using the previously defined list (SetupInputFrame()).
					@returns(Frame that contains the values of all specified inputs.)
				}
				function ReadFrame(): TInputFrame;
				{**
					Define the outputs that should be set using the next frame based out transfer.
					@param(Outputs Collection of outputs.)
				}
				procedure SetupOutputFrame(Outputs: TIOutputList);
				{**
					Write multiple outputs at once. All outputs specified using SetupOutputFrame() are modified using
					the values in the frame.
					@param(Frame)
				}
				procedure WriteFrame(Frame: TOutputFrame);
				{**
					Create OutputFrame instance specific to this device.
					@returns(New instance of OutputFrame.)
				}
				function CreateOutputFrame(): TOutputFrame;
				{** List of single ended analog inputs.}
				property SingleEndedInputs: TAnalogInputList read _SingleEnded;
				{** List of differential analog inputs.}
				property DifferentialInputs: TAnalogInputList read _Differential;
				{** List of analog outputs.}
				property AnalogOutputs: TAnalogOutputList read _AnalogOutputs;
				{** List of digital ports.}
				property DigitalPorts: TDigitalPortList read _DigitalPorts;
				{** List of counters.}
				property Counters: TCounterList read _Counters;
				{** List of triggers.}
				property Triggers: TTriggerList read _Triggers;
				{** List of LEDs.}
				property Leds: TLedList read _Leds;
				{** List of current sources.}
				property CurrentSources: TCurrentSourceList read _CurrentSources;
				function ToString(): string; override;
		end;
		TDeviceList = array of TDevice;

		{** Class that is responsible for all device independent functionality.}
		TLibraryInterface = class
			protected
				class function GetApiVersion(): ansistring; static;
			public
				class property ApiVersion: ansistring read GetApiVersion;
				class function Enumerate(DevType: TDeviceType): TDeviceList;
				class function EnumerateNetwork(Hostname: ansistring; Port: longint; DevType: TDeviceType): TDeviceList;
		end;
		
	var
		{** List of all available items for device-enumeration.}
		DeviceType : TDeviceTypeList;

	const
		{** Valid flags to use with Device.ResetPeripherals()}
		FlagResetInputFifo : longint = $00000001;
	
implementation
		
	const
		{$ifdef WIN32}
			{$define FilenameSet}
			ExternalFilename = 'cebomsr-1.5-x86.dll';
		{$endif}
		{$ifdef WIN64}
			{$define FilenameSet}
			ExternalFilename = 'cebomsr-1.5-x86_64.dll';
		{$endif}
		{$ifndef FilenameSet}
			{$fatal Filename for external Library has to be set}
		{$endif}
		{$ifdef MSWINDOWS}
			CRLF = #13#10;
		{$endif}
		{$ifdef LINUX}
			CRLF = #10;
		{$endif}

		CeboMsrTypeAll : CeboMsrType = $ceb00000;
		CeboMsrTypeUsb : CeboMsrType = $ceb00001;
		CeboMsrTypeCeboLC : CeboMsrType = $ceb00010;
		CeboMsrTypeCeboStick : CeboMsrType = $ceb00011;
		CeboMsrTypeCeboDFN : CeboMsrType = $ceb00012;

		MaxIdentLength : longint = 1024;

		CeboParameterSingleEndedCount : longword = 0;
		CeboParameterDifferentialCount : longword = 1;
		CeboParameterAnalogOutputCount : longword = 2;
		CeboParameterDigitalPortCount : longword = 3;
		CeboParameterCounterCount : longword = 4;
		CeboParameterTriggerCount : longword = 5;
		CeboParameterMaxInputFrames : longword = 6;
		CeboParameterCurrentSourceCount : longword = 7;
		CeboParameterLedCount : longword = 8;

		CeboMsrIoIdDigitalPort0 : CeboMsrIo = $ceb01000;
		CeboMsrIoIdDigitalPort1 : CeboMsrIo = $ceb01001;
		CeboMsrIoIdDigitalPort2 : CeboMsrIo = $ceb01002;
		CeboMsrIoIdDigitalPort3 : CeboMsrIo = $ceb01003;
		CeboMsrIoIdDigitalPort4 : CeboMsrIo = $ceb01004;
		CeboMsrIoIdDigitalPort5 : CeboMsrIo = $ceb01005;
		CeboMsrIoIdDigitalPort6 : CeboMsrIo = $ceb01006;
		CeboMsrIoIdDigitalPort7 : CeboMsrIo = $ceb01007;

		CeboMsrIoIdSingleEnded0 : CeboMsrIo = $ceb01100;
		CeboMsrIoIdSingleEnded1 : CeboMsrIo = $ceb01101;
		CeboMsrIoIdSingleEnded2 : CeboMsrIo = $ceb01102;
		CeboMsrIoIdSingleEnded3 : CeboMsrIo = $ceb01103;
		CeboMsrIoIdSingleEnded4 : CeboMsrIo = $ceb01104;
		CeboMsrIoIdSingleEnded5 : CeboMsrIo = $ceb01105;
		CeboMsrIoIdSingleEnded6 : CeboMsrIo = $ceb01106;
		CeboMsrIoIdSingleEnded7 : CeboMsrIo = $ceb01107;
		CeboMsrIoIdSingleEnded8 : CeboMsrIo = $ceb01108;
		CeboMsrIoIdSingleEnded9 : CeboMsrIo = $ceb01109;
		CeboMsrIoIdSingleEnded10 : CeboMsrIo = $ceb0110a;
		CeboMsrIoIdSingleEnded11 : CeboMsrIo = $ceb0110b;
		CeboMsrIoIdSingleEnded12 : CeboMsrIo = $ceb0110c;
		CeboMsrIoIdSingleEnded13 : CeboMsrIo = $ceb0110d;
		CeboMsrIoIdSingleEnded14 : CeboMsrIo = $ceb0110e;
		CeboMsrIoIdSingleEnded15 : CeboMsrIo = $ceb0110f;

		CeboMsrIoIdDifferential0 : CeboMsrIo = $ceb01200;
		CeboMsrIoIdDifferential1 : CeboMsrIo = $ceb01201;
		CeboMsrIoIdDifferential2 : CeboMsrIo = $ceb01202;
		CeboMsrIoIdDifferential3 : CeboMsrIo = $ceb01203;
		CeboMsrIoIdDifferential4 : CeboMsrIo = $ceb01204;
		CeboMsrIoIdDifferential5 : CeboMsrIo = $ceb01205;
		CeboMsrIoIdDifferential6 : CeboMsrIo = $ceb01206;
		CeboMsrIoIdDifferential7 : CeboMsrIo = $ceb01207;

		CeboMsrIoIdAnalogOutput0 : CeboMsrIo = $ceb01300;
		CeboMsrIoIdAnalogOutput1 : CeboMsrIo = $ceb01301;

		CeboMsrRange0 : CeboMsrRange = $ceb01400;
		CeboMsrRange1 : CeboMsrRange = $ceb01401;
		CeboMsrRange2 : CeboMsrRange = $ceb01402;
		CeboMsrRange3 : CeboMsrRange = $ceb01403;
		CeboMsrRangeInvalid : CeboMsrRange = $ceb014ff;

		CeboMsrIoIdCounter0 : CeboMsrIo = $ceb01500;

		CeboMsrIoIdTrigger0 : CeboMsrIo = $ceb01600;

		CeboMsrCurrentSource0 : CeboMsrCurrentSource = $ceb01700;
		CeboMsrCurrentSource1 : CeboMsrCurrentSource = $ceb01701;

		CeboMsrLedIndicator : CeboMsrLed = $ceb01800;

{*******************************************************}
{       API-Lib Instantiation                           }
{*******************************************************}

/// Error handling
	function CeboMsrGetErrorLength(Index: CeboMsrErrorIndex): longint; cdecl; external ExternalFilename;
	procedure CeboMsrGetError(Index: CeboMsrErrorIndex; Buffer: pansichar); cdecl; external ExternalFilename;

/// Host version information
	function CeboMsrGetApiVersion(Version: pansichar): CeboMsrErrorIndex; cdecl; external ExternalFilename;

/// Enumeration
	function CeboMsrBeginEnumerate(var Count: longint; MsrType : CeboMsrType): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetIdentifier(Id: longint; Ident: pansichar; var MsrType : CeboMsrType; Version: pansichar): CeboMsrErrorIndex; cdecl; external ExternalFilename;

/// Access
	function CeboMsrOpen(Ident: pansichar; var Handle: CeboMsrHandle): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrClose(Handle: CeboMsrHandle): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrResetDevice(Handle: CeboMsrHandle): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrResetPeripherals(Handle: CeboMsrHandle; Mask: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;

/// Box specific
	function CeboMsrGetDeviceParameter(Handle: CeboMsrHandle; Parameter: longint; var MsrValue: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrEnumerateRanges(Handle: CeboMsrHandle; IoId: CeboMsrIo; Id: longint; var RangeId: CeboMsrRange): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetRangeMinMax(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange; var MinValue: single; var MaxValue: single): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetDigitalPortIoCount(Handle: CeboMsrHandle; IoId: CeboMsrIo; var IoCount: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetPeripheralName(Handle: CeboMsrHandle; Id: longint; Name: pansichar): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetDigitalPortIoName(Handle: CeboMsrHandle; IoId: CeboMsrIo; Io: longint; Name: pansichar): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetDefaultAnalogInputInterChannelDelay(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange; var DefaultInterChannelDelay: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetMinAnalogInputInterChannelDelay(Handle: CeboMsrHandle; IoId: CeboMsrIo; var MinInterChannelDelay: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetSerialNumber(Handle: CeboMsrHandle; Buffer: pansichar): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetTemperature(Handle: CeboMsrHandle; var Temperature: single): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetLed(Handle: CeboMsrHandle; Led: CeboMsrLed; State: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetReferenceCurrent(Handle: CeboMsrHandle; Source: CeboMsrCurrentSource; var Current: single): CeboMsrErrorIndex; cdecl; external ExternalFilename;

/// Configuration
	function CeboMsrSetDigitalPortOutputEnableMask(Handle: CeboMsrHandle; IoId: CeboMsrIo; Mask: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetAnalogInputParameters(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange; InterChannelDelay: longint; var CorrectedInterChannelDelay: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetAnalogInputParameters(Handle: CeboMsrHandle; IoId: CeboMsrIo; var RangeId: CeboMsrRange; var InterChannelDelay: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetTriggerConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; TriggerConfigId: CeboMsrTriggerConfigId): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetTriggerConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; var TriggerConfigId: CeboMsrTriggerConfigId): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetTriggerEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; Enable: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetTriggerEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; var Enable: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrResetCounter(Handle: CeboMsrHandle; IoId: CeboMsrIo): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetCounterConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; CounterConfigId: CeboMsrCounterConfigId): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetCounterConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; var CounterConfigId: CeboMsrCounterConfigId): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetCounterEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; Enable: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrGetCounterEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; var Enable: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;

/// Input specific
	function CeboMsrCalculateMaxBufferedInputFrames(Handle: CeboMsrHandle; var FrameCount: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrSetupInputFrame(Handle: CeboMsrHandle; var Ios: CeboMsrIo; count: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrStartDataAcquisition(Handle: CeboMsrHandle; ExternalStarted: longint; FrameRate: single; FrameCount: longint; var CorrectedFrameRate: single): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrStopDataAcquisition(Handle: CeboMsrHandle): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrReadBlocking(Handle: CeboMsrHandle; Vbuffer: VoltageArray; Buffer: DigitalArray; FrameCount: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrReadNonBlocking(Handle: CeboMsrHandle; Vbuffer: VoltageArray; Buffer: DigitalArray; var Transferred: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrReadDigitalPort(Handle: CeboMsrHandle; IoId: CeboMsrIo; var MsrValue: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrReadAnalogInput(Handle: CeboMsrHandle; IoId: CeboMsrIo; var MsrValue: single): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrReadFrame(Handle: CeboMsrHandle; Vbuffer: VoltageArray; Buffer: DigitalArray): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrReadCounter(Handle: CeboMsrHandle; IoId: CeboMsrIo; var Counter: int64): CeboMsrErrorIndex; cdecl; external ExternalFilename;

/// Output specific
	function CeboMsrSetupOutputFrame(Handle: CeboMsrHandle; var Ios: CeboMsrIo; Count: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrWriteBlocking(Handle: CeboMsrHandle; Vbuffer: VoltageArray; Buffer: DigitalArray; FrameCount: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrWriteNonBlocking(Handle: CeboMsrHandle; Vbuffer: VoltageArray; Buffer: DigitalArray; FrameCount: longint; var Transferred: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrWriteDigitalPort(Handle: CeboMsrHandle; IoId: CeboMsrIo; MsrValue: longint): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrWriteAnalogOutput(Handle: CeboMsrHandle; IoId: CeboMsrIo; MsrValue: single): CeboMsrErrorIndex; cdecl; external ExternalFilename;
	function CeboMsrWriteFrame(Handle: CeboMsrHandle; Vbuffer: VoltageArray; Buffer: DigitalArray): CeboMsrErrorIndex; cdecl; external ExternalFilename;

{*******************************************************}
{       API-Lib Wrappers                                }
{*******************************************************}

	class procedure TSystemConnector.FillSArray(var arr: TStringType);
		{fills whole char-array with NULL-Values to initialize it}
		var i: longint;
	begin
		for i := 1 to MaxIdentLength do
			arr[i] := #0;
	end;

	class procedure TSystemConnector.CheckError(ErrorCode: CeboMsrErrorIndex);
		var
			ErrorClass : longword;
			SError : TStringType;
			Error : pansichar;
	begin
		if (ErrorCode <> 0) then
		begin
			FillSArray(SError);
			Error := @SError[1];
			ErrorClass := ErrorCode shr 24;
			CeboMsrGetError(ErrorCode, Error);
			case ErrorClass of
				$80 : raise EArgumentException.Create(string(Error));
				$40 : raise EIndexOutOfRangeException.Create(string(Error));
				$20 : raise EInvalidOperationException.Create(string(Error));
				$10 : raise EIOException.Create(string(Error));
			else
				raise EApplicationException('(internal) Unhandled error: ' + string(Error));
			end;
		end;
	end;

	class function TSystemConnector.GetApiVersion(): ansistring;
		var
			SApiVersion : TStringType;
			PCApiVersion : pansichar;
	begin
		FillSArray(SApiVersion);
		PCApiVersion := @SApiVersion[1];
		CheckError(CeboMsrGetApiVersion(PCApiVersion));
		GetApiVersion := PCApiVersion;
	end;
	
	class function TSystemConnector.BeginEnumerate(MsrType: CeboMsrType): longint;
		var Enumerate: longint;
	begin
		Enumerate := 0;
		CheckError(CeboMsrBeginEnumerate(Enumerate,MsrType));
		BeginEnumerate := Enumerate;
	end;

	class function TSystemConnector.BeginEnumerateNetwork(Hostname: ansistring; Port: longint; MsrType: CeboMsrType): longint;
	begin
		BeginEnumerateNetwork := 0;
		raise ENotImplemented.Create('');
	end;

	class function TSystemConnector.GetIdentifier(Id: longint; var MsrType: CeboMsrType; var Version: ansistring): ansistring;
		var
			SR, VR : TStringType;
			PCR : pansichar;
			PVersion: pansichar;
	begin
		FillSArray(SR);
		PCR := @SR[1];
		FillSArray(VR);
		PVersion := @VR[1];
		CheckError(CeboMsrGetIdentifier(Id, PCR, MsrType, PVersion));
		Version := PVersion;
		GetIdentifier := PCR;
	end;

	class function TSystemConnector.Open(Ident: ansistring):CeboMsrHandle;
		var 
			OpenVar: CeboMsrHandle;
			PIdent: pansichar;
	begin
		OpenVar := 0;
		PIdent := pansichar(Ident);
		CheckError(CeboMsrOpen(PIdent,OpenVar));
		Open := OpenVar;
	end;

	class procedure TSystemConnector.Close(Handle: CeboMsrHandle);
	begin
		CheckError(CeboMsrClose(Handle));
	end;

	class procedure TSystemConnector.ResetDevice(Handle: CeboMsrHandle);
	begin
		CheckError(CeboMsrResetDevice(Handle));
	end;

	class procedure TSystemConnector.ResetPeripherals(Handle: CeboMsrHandle; Mask: longint);
	begin
		CheckError(CeboMsrResetPeripherals(Handle,Mask));
	end;

	class function TSystemConnector.GetDeviceParameter(Handle: CeboMsrHandle; Parameter: longint): longint;
		var DeviceParameter: longint;
	begin
		DeviceParameter := 0;
		CheckError(CeboMsrGetDeviceParameter(Handle, Parameter, DeviceParameter));
		GetDeviceParameter := DeviceParameter;
	end;

	class function TSystemConnector.GetDigitalPortIoCount(Handle: CeboMsrHandle; IoId: CeboMsrIo): longint;
		var Count: longint;
	begin
		Count := 0;
		CheckError(CeboMsrGetDigitalPortIoCount(Handle, IoId, Count));
		GetDigitalPortIoCount := Count;
	end;

	class function TSystemConnector.GetPeripheralName(Handle: CeboMsrHandle; IoId: CeboMsrIo): ansistring;
		var
			SName : TStringType;
			PCName : pansichar;
	begin
		FillSArray(SName);
		PCName := @SName[1];
		CheckError(CeboMsrGetPeripheralName(Handle, IoId, PCName));
		GetPeripheralName := PCName;
	end;

	class function TSystemConnector.GetDigitalPortIoName(Handle: CeboMsrHandle; IoId: CeboMsrIo; Io: longint): ansistring;
		var
			SName : TStringType;
			PCName : pansichar;
	begin
		FillSArray(SName);
		PCName := @SName[1];
		CheckError(CeboMsrGetDigitalPortIoName(Handle, IoId, Io, PCName));
		GetDigitalPortIoName := PCName;
	end;

	class function TSystemConnector.EnumerateRanges(Handle: CeboMsrHandle; IoId: CeboMsrIo; Id: longint): CeboMsrRange;
		var Ranges: CeboMsrRange;
	begin
		Ranges := 0;
		CheckError(CeboMsrEnumerateRanges(Handle, IoId, Id, Ranges));
		EnumerateRanges := Ranges;
	end;

	class function TSystemConnector.GetRangeMinMax(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange):TRangeMinMax;
		var
			MinValue: single;
			MaxValue: single;
	begin
		MinValue := 0;
		MaxValue := 0;
		CheckError(CeboMsrGetRangeMinMax(Handle, IoId, RangeId, MinValue, MaxValue));
		GetRangeMinMax[1] := MinValue;
		GetRangeMinMax[2] := MaxValue;
	end;

	class function TSystemConnector.GetDefaultAnalogInputInterChannelDelay(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange):longint;
		var Return: longint;
	begin
		Return := 0;
		CheckError(CeboMsrGetDefaultAnalogInputInterChannelDelay(Handle, IoId, RangeId, Return));
		GetDefaultAnalogInputInterChannelDelay := Return;
	end;

	class function TSystemConnector.GetMinAnalogInputInterChannelDelay(Handle: CeboMsrHandle; IoId: CeboMsrIo):longint;
		var Return: longint;
	begin
		Return := 0;
		CheckError(CeboMsrGetMinAnalogInputInterChannelDelay(Handle, IoId, Return));
		GetMinAnalogInputInterChannelDelay := Return;
	end;

	class function TSystemConnector.GetSerialNumber(Handle: CeboMsrHandle): ansistring;
		var 
			SNumber : TStringType;
			PCNumber : pansichar;
	begin
		FillSArray(SNumber);
		PCNumber := @SNumber[1];
		CheckError(CeboMsrGetSerialNumber(Handle, PCNumber));
		GetSerialNumber := PCNumber;
	end;

	class function TSystemConnector.GetTemperature(Handle: CeboMsrHandle): single;
		var Temperature: single;
	begin
		Temperature := 0.0;
		CheckError(CeboMsrGetTemperature(Handle, Temperature));
		GetTemperature := Temperature;
	end;

	class procedure TSystemConnector.SetLed(Handle: CeboMsrHandle; IoId: CeboMsrIo; State: boolean);
		var IState: longint;
	begin
		IState := 0;
		if State = true then IState := 1 else IState := 0;
		CheckError(CeboMsrSetLed(Handle,IoId,IState));
	end;

	class function TSystemConnector.GetReferenceCurrent(Handle: CeboMsrHandle; IoId: CeboMsrIo): single;
		var ReferenceCurrent: single;
	begin
		ReferenceCurrent := 0.0;
		CheckError(CeboMsrGetReferenceCurrent(Handle, IoId, ReferenceCurrent));
		GetReferenceCurrent := ReferenceCurrent;
	end;

	class procedure TSystemConnector.SetDigitalPortOutputEnableMask(Handle: CeboMsrHandle; IoId: CeboMsrIo; Mask: longint);
	begin
		CheckError(CeboMsrSetDigitalPortOutputEnableMask(Handle, IoId, Mask));
	end;

	class function TSystemConnector.SetAnalogInputParameters(Handle: CeboMsrHandle; IoId: CeboMsrIo; RangeId: CeboMsrRange; InterChannelDelay: longint): longint;
		var Parameters: longint;
	begin
		Parameters := 0;
		CheckError(CeboMsrSetAnalogInputParameters(Handle, IoId, RangeId, InterChannelDelay, Parameters));
		SetAnalogInputParameters := Parameters;
	end;

	class function TSystemConnector.GetAnalogInputParameters(Handle: CeboMsrHandle; IoId: CeboMsrIo; InterChannelDelay: longint): CeboMsrRange;
		var Range: CeboMsrRange;
	begin
		Range := 0;
		CheckError(CeboMsrGetAnalogInputParameters(Handle, IoId, Range, InterChannelDelay));
		GetAnalogInputParameters := Range;
	end;

	class procedure TSystemConnector.SetTriggerEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; State: boolean);
		var IState: longint;
	begin
		if State = true then IState := 1 else IState := 0;
		CheckError(CeboMsrSetTriggerEnable(Handle,IoId,IState));
	end;

	class function TSystemConnector.GetTriggerEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo): boolean;
		var State: longint;
	begin
		State := 0;
		CheckError(CeboMsrGetTriggerEnable(Handle, IoId, State));
		GetTriggerEnable := (State = 1);
	end;

	class procedure TSystemConnector.SetTriggerConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; TriggerConfigId: CeboMsrTriggerConfigId);
	begin
		CheckError(CeboMsrSetTriggerConfig(Handle, IoId, TriggerConfigId));
	end;

	class function TSystemConnector.GetTriggerConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo): CeboMsrTriggerConfigId;
		var Config: CeboMsrTriggerConfigId;
	begin
		Config := 0;
		CheckError(CeboMsrGetTriggerConfig(Handle, IoId, Config));
		GetTriggerConfig := Config;
	end;

	class procedure TSystemConnector.ResetCounter(Handle: CeboMsrHandle; IoId: CeboMsrIo);
	begin
		CheckError(CeboMsrResetCounter(Handle, IoId));
	end;

	class procedure TSystemConnector.SetCounterEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo; State: boolean);
		var IState: longint;
	begin
		if State = true then IState := 1 else IState := 0;
		CheckError(CeboMsrSetCounterEnable(Handle, IoId, IState));
	end;

	class function TSystemConnector.GetCounterEnable(Handle: CeboMsrHandle; IoId: CeboMsrIo): boolean;
		var State: longint;
	begin
		State := 0;
		CheckError(CeboMsrGetCounterEnable(Handle, IoId, State));
		GetCounterEnable := (State = 1);
	end;

	class procedure TSystemConnector.SetCounterConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo; CounterConfigId: CeboMsrCounterConfigId);
	begin
		CheckError(CeboMsrSetCounterConfig(Handle, IoId, CounterConfigId));
	end;

	class function TSystemConnector.GetCounterConfig(Handle: CeboMsrHandle; IoId: CeboMsrIo): CeboMsrCounterConfigId;
		var Config: CeboMsrCounterConfigId;
	begin
		Config := 0;
		CheckError(CeboMsrGetCounterConfig(Handle, IoId, Config));
		GetCounterConfig := Config;
	end;

	class function TSystemConnector.CalculateMaxBufferedInputFrames(Handle: CeboMsrHandle): longint;
		var Frames: longint;
	begin
		Frames := 0;
		CheckError(CeboMsrCalculateMaxBufferedInputFrames(Handle, Frames));
		CalculateMaxBufferedInputFrames := Frames;
	end;

	class procedure TSystemConnector.SetupInputFrame(Handle: CeboMsrHandle; var Ios: IoArray);
	begin
		CheckError(CeboMsrSetupInputFrame(Handle, Ios[0], length(Ios)));
	end;

	class function TSystemConnector.StartDataAcquisition(Handle: CeboMsrHandle; ExternalStarted: boolean; FrameRate: single; FrameCount: longint): single;
		var
			Started : longint;
			Float: single;
	begin
		Started := 0;
		Float := 0.0;
		if ExternalStarted = true then Started := 1 else Started := 0;
		CheckError(CeboMsrStartDataAcquisition(Handle, Started, FrameRate, FrameCount, Float));
		StartDataAcquisition := Float;
	end;

	class procedure TSystemConnector.StopDataAcquisition(Handle: CeboMsrHandle);
	begin
		CheckError(CeboMsrStopDataAcquisition(Handle));
	end;

	class procedure TSystemConnector.ReadBlocking(Handle: CeboMsrHandle; var VBuffer: VoltageArray; var Buffer: DigitalArray; FrameCount: longint);
	begin
		CheckError(CeboMsrReadBlocking(Handle, VBuffer, Buffer, FrameCount));
	end;

	class function TSystemConnector.ReadNonBlocking(Handle: CeboMsrHandle; VBuffer: VoltageArray; Buffer: DigitalArray): longint;
		var Return: longint;
	begin
		Return := 0;
		CheckError(CeboMsrReadNonBlocking(Handle, VBuffer, Buffer, Return));
		ReadNonBlocking := Return;
	end;

	class function TSystemConnector.ReadDigitalPort(Handle: CeboMsrHandle; IoId: CeboMsrIo): longint;
		var Return: longint;
	begin
		Return := 0;
		CheckError(CeboMsrReadDigitalPort(Handle, IoId, Return));
		ReadDigitalPort := Return;
	end;

	class function TSystemConnector.ReadAnalogInput(Handle: CeboMsrHandle; IoId: CeboMsrIo): single;
		var Return: single;
	begin
		Return := 0.0;
		CheckError(CeboMsrReadAnalogInput(Handle, IoId, Return));
		ReadAnalogInput := Return;
	end;

	class procedure TSystemConnector.ReadFrame(Handle: CeboMsrHandle; VBuffer: VoltageArray; Buffer: DigitalArray);
	begin
		CheckError(CeboMsrReadFrame(Handle, VBuffer, Buffer));
	end;

	class function TSystemConnector.ReadCounter(Handle: CeboMsrHandle; IoId: CeboMsrIo): int64;
		var Return: int64;
	begin
		Return := 0;
		CheckError(CeboMsrReadCounter(Handle, IoId, Return));
		ReadCounter := Return;
	end;

	class procedure TSystemConnector.SetupOutputFrame(Handle: CeboMsrHandle; Ios: IoArray);
	begin
		CheckError(CeboMsrSetupOutputFrame(Handle, Ios[0], length(Ios)));
	end;

	class procedure TSystemConnector.WriteDigitalPort(Handle: CeboMsrHandle; IoId: CeboMsrIo; MsrValue: longint);
	begin
		CheckError(CeboMsrWriteDigitalPort(Handle, IoId, MsrValue));
	end;

	class procedure TSystemConnector.WriteAnalogOutput(Handle: CeboMsrHandle; IoId: CeboMsrIo; MsrValue: single);
	begin
		CheckError(CeboMsrWriteAnalogOutput(Handle, IoId, MsrValue));
	end;

	class procedure TSystemConnector.WriteFrame(Handle: CeboMsrHandle; VBuffer: VoltageArray; Buffer: DigitalArray);
	begin
		CheckError(CeboMsrWriteFrame(Handle, VBuffer, Buffer));
	end;

{*******************************************************}
{       Class Methods                                   }
{*******************************************************}

//
//// TRange
//
	constructor TRange.Create(RngeId: CeboMsrRange; MinVal, MaxVal: single; DefInterChannelDelay: longint);
	begin
		_RangeId := RngeId;
		_MinValue := MinVal;
		_MaxValue := MaxVal;
		_DefaultInterChannelDelay := DefInterChannelDelay;
	end;

	function TRange.ToString(): string;
	begin
		ToString := 'Range: ' + FloatToStr(_MinValue) + 'V - ' + FloatToStr(_MaxValue) + 'V';
	end;

//
//// TAnalogInput
//
	function TAnalogInput.GetRange(): TRange;
		var
			Dummy: longint;
			RangeId: longword;
			N: longword;
			I: longword;
	begin
		Dummy := 0;
		RangeId := TSystemConnector.GetAnalogInputParameters(_Device.Handle, _InterfaceId, dummy);
		N := Length(_SupportedRanges);
		I := 0;
		while (I < N) and (_SupportedRanges[I].RangeId <> RangeId) do begin
			//search for Range with InterfaceId
			i := i + 1;
		end;
		GetRange := _SupportedRanges[I];
	end;
	
	function TAnalogInput.GetInterChannelDelay(): longint;
	begin
		GetInterChannelDelay := 0;
		TSystemConnector.GetAnalogInputParameters(_Device.Handle, _InterfaceId, GetInterChannelDelay);
	end;

	function TAnalogInput.GetVoltageValue(): single;
	begin
		GetVoltageValue := TSystemConnector.ReadAnalogInput(_Device.Handle, _InterfaceId);
	end;
		
	constructor TAnalogInput.Create(Device: TDevice; IfId: CeboMsrIo; Styl: TStyles; Idx: longint);
		var
			I: longint;
			RangeId: CeboMsrRange;
			MinMax: TRangeMinMax;
			DefaultICD: longint;
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Style := Styl;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);

		I := 0;
		RangeId := TSystemConnector.EnumerateRanges(Device.Handle, IfId, I);
		while RangeId <> CeboMsrRangeInvalid do begin
			MinMax := TSystemConnector.GetRangeMinMax(Device.Handle, IfId, RangeId);
			DefaultICD := TSystemConnector.GetDefaultAnalogInputInterChannelDelay(Device.Handle, IfId, RangeId);
			SetLength(_SupportedRanges,Length(_SupportedRanges)+1);
			_SupportedRanges[Length(_SupportedRanges) -1] := TRange.Create(RangeId, MinMax[1], MinMax[2], DefaultICD);
			I := I + 1;
			RangeId := TSystemConnector.EnumerateRanges(Device.Handle, IfId, I);
		end;
		_MinInterChannelDelay := TSystemConnector.GetMinAnalogInputInterChannelDelay(Device.Handle, IfId);
	end;

	function TAnalogInput.GetDefaultInterChannelDelay(ARange: TRange): longint;
	begin
		GetDefaultInterChannelDelay := ARange.DefaultInterChannelDelay;
	end;
	
	procedure TAnalogInput.SetParameters(ARange: TRange);
	begin
		TSystemConnector.SetAnalogInputParameters(_Device.Handle, _InterfaceId, ARange.RangeId,-1);
	end;
	
	function TAnalogInput.SetParameters(ARange: TRange; InterChannelDelay: longint):longint;
	begin
		SetParameters := TSystemConnector.SetAnalogInputParameters(_Device.Handle, _InterfaceId, Range.RangeId, InterChannelDelay);
	end;
	
	function TAnalogInput.ToString(): string;
	begin
		if _Style = Differential then begin
			ToString := 'Differential ' + IntToStr(_index);
		end else begin
			ToString := 'SingleEnded ' + IntToStr(_index);
		end;
	end;

//
//// TAnalogOutput
//

	function TAnalogOutput.GetRange(): TRange;
	begin
		GetRange := _SupportedRanges[0];
	end;

	constructor TAnalogOutput.Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
		var
			I: longint;
			RangeId: CeboMsrRange;
			MinMax: TRangeMinMax;
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);

		I := 0;
		RangeId := TSystemConnector.EnumerateRanges(Device.Handle, IfId, I);
		while RangeId <> CeboMsrRangeInvalid do begin
			MinMax := TSystemConnector.GetRangeMinMax(Device.Handle, IfId, RangeId);
			SetLength(_SupportedRanges,Length(_SupportedRanges)+1);
			_SupportedRanges[Length(_SupportedRanges) -1] := TRange.Create(
				RangeId,
				MinMax[1],
				MinMax[2],
				0
			);
			I := I + 1;
			RangeId := TSystemConnector.EnumerateRanges(Device.Handle, IfId, I);
		end;
	end;

	procedure TAnalogOutput.WriteValue(Value: single);
	begin
		TSystemConnector.WriteAnalogOutput(_Device.Handle, _InterfaceId, Value);
	end;

	function TAnalogOutput.ToString(): string;
	begin
		ToString := 'AnalogOutput ' + IntToStr(_Index);
	end;

	procedure TAnalogOutput.SetParameters(Range: TRange);
	begin
		// No device currently supports this, included for future functionality.
	end;


//
//// TDigitalPort
//

	constructor TDigitalPort.Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
		var I: longint;
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);
		_IoCount := TSystemConnector.GetDigitalPortIoCount(Device.Handle, IfId);
		SetLength(_IoNames,_IoCount);
		I := 0;
		while I < _IoCount do begin
			_IoNames[I] := TSystemConnector.GetDigitalPortIoName(Device.Handle, IfId, I);
			I := I + 1;
		end;
	end;

	procedure TDigitalPort.SetOutputEnableMask(Mask: longint);
	begin
		TSystemConnector.SetDigitalPortOutputEnableMask(_Device.Handle, _InterfaceId, Mask);
	end;
	
	function TDigitalPort.GetValue(): longint;
	begin
		GetValue := TSystemConnector.ReadDigitalPort(_Device.Handle, _InterfaceId);
	end;
	
	procedure TDigitalPort.SetValue(val: longint);
	begin
		TSystemConnector.WriteDigitalPort(_Device.Handle, _InterfaceId, val);
	end;
	
	function TDigitalPort.GetIoName(Io: longint): ansistring;
	begin
		GetIoName := _IoNames[Io];
	end;
	
	function TDigitalPort.ToString(): string;
	begin
		ToString := 'DigitalPort ' + IntToStr(_Index);
	end;

//
//// TCounter
//

	constructor TCounter.Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);
	end;

	procedure TCounter.SetEnabled(State: boolean);
	begin
		TSystemConnector.SetCounterEnable(_Device.Handle, _InterfaceId, State);
	end;
	
	function TCounter.GetEnabled(): boolean;
	begin
		GetEnabled := TSystemConnector.GetCounterEnable(_Device.Handle, _InterfaceId);
	end;

	procedure TCounter.SetConfig(Conf: TCounterConfig);
	begin
		TSystemConnector.SetCounterConfig(_Device.Handle, _InterfaceId, ord(Conf));
	end;
	
	function TCounter.GetConfig(): TCounterConfig;
	begin
		GetConfig := TCounterConfig(TSystemConnector.GetCounterConfig(_Device.Handle, _InterfaceId));
	end;

	function TCounter.GetValue(): Int64;
	begin
		GetValue := TSystemConnector.ReadCounter(_Device.Handle, _InterfaceId);
	end;

	procedure TCounter.Reset();
	begin
		TSystemConnector.ResetCounter(_Device.Handle, _InterfaceId);
	end;
	
	function TCounter.ToString(): string;
	begin
		ToString := 'Counter ' + IntToStr(_Index);
	end;

//
//// TTrigger
//

	constructor TTrigger.Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);
	end;

	procedure TTrigger.SetEnabled(State: boolean);
	begin
		TSystemConnector.SetTriggerEnable(_Device.Handle, _InterfaceId, State);
	end;
	
	function TTrigger.GetEnabled(): boolean;
	begin
		GetEnabled := TSystemConnector.GetTriggerEnable(_Device.Handle, _InterfaceId);
	end;

	procedure TTrigger.SetConfig(Conf: TTriggerConfig);
	begin
		TSystemConnector.SetTriggerConfig(_Device.Handle, _InterfaceId, ord(Conf));
	end;
	
	function TTrigger.GetConfig(): TTriggerConfig;
	begin
		GetConfig := TTriggerConfig(TSystemConnector.GetTriggerConfig(_Device.Handle, _InterfaceId));
	end;
	
	function TTrigger.ToString(): string;
	begin
		ToString := 'Trigger ' + IntToStr(_Index);
	end;

//
//// TLed
//

	constructor TLed.Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);
	end;

	procedure TLed.SetEnabled(State: boolean);
	begin
		TSystemConnector.SetLed(_Device.Handle, _InterfaceId, State);
	end;
	
	function TLed.ToString(): string;
	begin
		ToString := 'LED ' + string(_Name);
	end;

//
//// TCurrentSource
//

	constructor TCurrentSource.Create(Device: TDevice; IfId: CeboMsrIo; Idx: longint);
	begin
		_Device := Device;
		_InterfaceId := IfId;
		_Index := Idx;
		_Name := TSystemConnector.GetPeripheralName(Device.Handle, IfId);
	end;

	function TCurrentSource.GetReference(): single;
	begin
		GetReference := TSystemConnector.GetReferenceCurrent(_Device.Handle, _InterfaceId);
	end;
	
	function TCurrentSource.ToString(): string;
	begin
		ToString := 'CurrentSource ' + string(_Name);
	end;

//
//// TInputFrame
//

	constructor TInputFrame.Create(VValues: VoltageArray; Values: DigitalArray; SingleEndedOffset, SingleEndedCount, DifferentialOffset, DifferentialCount, DigitalPortOffset, DigitalPortCount, CounterOffset, CounterCount, TriggerOffset, TriggerCount: longint);
	begin
		_VValues := VValues;
		_Values := Values;
		_SingleEndedOffset := SingleEndedOffset;
		_SingleEndedCount := SingleEndedCount;
		_DifferentialOffset := DifferentialOffset;
		_DifferentialCount := DifferentialCount;
		_DigitalPortOffset := DigitalPortOffset;
		_DigitalPortCount := DigitalPortCount;
		_CounterOffset := CounterOffset;
		_CounterCount := CounterCount;
		_TriggerOffset := TriggerOffset;
		_TriggerCount := TriggerCount;
	end;
	
	function TInputFrame.GetSingleEnded(Index: longint): single;
	begin
		if ((Index < 0) or (Index >= _SingleEndedCount)) then begin
			//throw exception
			raise EIndexOutOfRangeException.Create('SingleEnded');
		end else begin
			GetSingleEnded := _VValues[_SingleEndedOffset + Index];
		end;
	end;
	
	function TInputFrame.GetDifferential(Index: longint): single;
	begin
		if ((Index < 0) or (Index >= _DifferentialCount)) then begin
			//throw exception
			raise EIndexOutOfRangeException.Create('Differential');
		end else begin
			GetDifferential := _VValues[_DifferentialOffset + Index];
		end;
	end;
	
	function TInputFrame.GetDigitalPort(Index: longint): longint;
	begin
		if ((Index < 0) or (Index >= _DigitalPortCount)) then begin
			//throw exception
			raise EIndexOutOfRangeException.Create('DigitalPort');
		end else begin
			GetDigitalPort := _Values[_DigitalPortOffset + Index];
		end;
	end;
	
	function TInputFrame.GetTrigger(Index: longint): boolean;
	begin
		if ((Index < 0) or (Index >= _TriggerCount)) then begin
			//throw exception
			raise EIndexOutOfRangeException.Create('Trigger');
		end else begin
			if _Values[_TriggerOffset + Index] = 0 then begin
				GetTrigger := false;
			end else begin
				GetTrigger := true;
			end;
		end;
	end;
	
	function TInputFrame.GetCounter(Index: longint): longint;
	begin
		if ((Index < 0) or (Index >= _CounterCount)) then begin
			//throw exception
			raise EIndexOutOfRangeException.Create('Counter');
		end else begin
			GetCounter := _Values[_CounterOffset + Index];
		end;
	end;
	
	function TInputFrame.ToString(): string;
		var I: longint;
	begin
		ToString := 'InputFrame: ' + CRLF;
		I := 0;
		while I < _SingleEndedCount do begin
			ToString := ToString + Format('SingleEnded #%0:d = %1:.2f V',[i, GetSingleEnded(I)]) + CRLF;
			I := I + 1;
		end;
		I := 0;
		while I < _DifferentialCount do begin
			ToString := ToString + Format('Differential #%0:d = %1:.2f V',[i, GetDifferential(I)]) + CRLF;
			I := I + 1;
		end;
		I := 0;
		while I < _DigitalPortCount do begin
			ToString := ToString + Format('DigitalPort #%0:d = %1:d (0x%1:8x)',[i, GetDigitalPort(I)]) + CRLF;
		end;
		I := 0;
		while I < _CounterCount do begin
			ToString := ToString + Format('Counter #%0:d = %1:d (0x%1:8x)',[i, GetCounter(I)]) + CRLF;
			I := I + 1;
		end;
		I := 0;
		while I < _TriggerCount do begin
			ToString := ToString + Format('Trigger #%0:d = %1:d',[i, GetTrigger(I)]) + CRLF;
			I := I + 1;
		end;
	end;

//
//// TOutputFrame
//

	constructor TOutputFrame.Create(VCount: longint; Count: longint);
	begin
		SetLength(_VValues,VCount);
		SetLength(_Values,Count);
	end;
	
	procedure TOutputFrame.SetDigitalPort(Index: longint; Value: longint);
	begin
		_Values[Index] := Value;
	end;
	
	procedure TOutputFrame.SetAnalogOutput(Index: longint; Value: single);
	begin
		_VValues[Index] := Value;
	end;
	
	function TOutputFrame.ToString(): string;
		var I: longint;
	begin
		ToString := 'OutputFrame:'+CRLF;
		I := 0;
		while I < Length(_VValues) do begin
			ToString := ToString + Format('AnalogOutput #%0:d = %1:.2f V',[I, _VValues[I]]);
			I := I + 1;
		end;
		I := 0;
		while I < Length(_Values) do begin
			ToString := ToString + Format('DigitalPort #%0:d = %1:d (0x%1:8x)',[I, _Values[I]]);
			I := I + 1;
		end;
	end;

//
//// TDeviceType
//
	constructor TDeviceType.Create(IfId: CeboMsrIo; Name: ansistring);
	begin
		_InterfaceId := IfId;
		_Name := Name;
	end;

	class function TDeviceType.GetDeviceTypeById(Id: CeboMsrType): TDeviceType;
		var fetched : boolean;
	begin
		GetDeviceTypeById := DeviceType.AllDevices;
		fetched := false;
		if Id = CeboMsrTypeAll then
		begin
			GetDeviceTypeById := DeviceType.AllDevices;
			fetched := true;
		end;
		if Id = CeboMsrTypeUsb then
		begin
			GetDeviceTypeById := DeviceType.Usb;
			fetched := true;
		end;
		if Id = CeboMsrTypeCeboLC then
		begin
			GetDeviceTypeById := DeviceType.CeboLC;
			fetched := true;
		end;
		if Id = CeboMsrTypeCeboStick then
		begin
			GetDeviceTypeById := DeviceType.CeboStick;
			fetched := true;
		end;
		if fetched = false then
		begin
			raise EIndexOutOfRangeException.Create('Unknown DeviceType Id');
			
		end;
	end;

//
//// TDeviceTypeList
//
	constructor TDeviceTypeList.Create();
	begin
		_AllDevices := TDeviceType.Create(CeboMsrTypeAll,'All');
		_Usb := TDeviceType.Create(CeboMsrTypeUsb,'Usb');
		_CeboLC := TDeviceType.Create(CeboMsrTypeCeboLC,'CeboLC');
		_CeboStick := TDeviceType.Create(CeboMsrTypeCeboStick,'CeboStick');
	end;

//
//// TDevice
//

	function TDevice.GetInputFrameFloatCount(): longint;
	begin
		GetInputFrameFloatCount := Length(_SingleEnded) + Length(_Differential);
	end;
	
	function TDevice.GetInputFrameIntCount(): longint;
	begin
		GetInputFrameIntCount := Length(_DigitalPorts) + Length(_Counters) + Length(_Triggers);
	end;
	
	function TDevice.GetOutputFrameFloatCount(): longint;
	begin
		GetOutputFrameFloatCount := Length(_AnalogOutputs);
	end;
	
	function TDevice.GetOutputFrameIntCount(): longint;
	begin
		GetOutputFrameIntCount := Length(_DigitalPorts);
	end;
	
	function TDevice.BuildInputFrame(VValues: VoltageArray; Values: DigitalArray; FrameNum: longint): TInputFrame;
		var
			VOffset: longint;
			Offset: longint;
	begin
		VOffset := FrameNum * InputFrameFloatCount;
		Offset := FrameNum * InputFrameIntCount;

		BuildInputFrame := TInputFrame.Create(
			VValues, Values,
			VOffset, Length(SingleEndedInputs),
			VOffset + Length(SingleEndedInputs), Length(DifferentialInputs),
			Offset, Length(DigitalPorts),
			Offset + Length(DigitalPorts), Length(Counters),
			Offset + Length(DigitalPorts) + Length(Counters), Length(Triggers)
		);
	end;
	
	function TDevice.GetSerNo(): ansistring;
	begin
		GetSerNo := TSystemConnector.GetSerialNumber(_Handle);
	end;
	
	function TDevice.GetTemp(): single;
	begin
		GetTemp := TSystemConnector.GetTemperature(_Handle);
	end;

	constructor TDevice.Create(Ident: ansistring; MsrType: TDeviceType; Version: ansistring);
	begin
		_Handle := $ffffffff;
		_MaxInputFrames := -1;
		_Ident := Ident;
		_MsrType := MsrType;
		_FirmwareVersion := Version;
	end;

	procedure TDevice.Open();
		var I: longword;
	begin
		_Handle := TSystemConnector.Open(_Ident);

		SetLength(_SingleEnded,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterSingleEndedCount) do begin
			SetLength(_SingleEnded, Length(_SingleEnded) + 1);
			_SingleEnded[I] := TAnalogInput.Create(
				self, CeboMsrIoIdSingleEnded0 + I, TStyles.SingleEnded, I
			);
			I := I + 1;
		end;

		SetLength(_Differential,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterDifferentialCount) do begin
			SetLength(_Differential, Length(_Differential) + 1);
			_Differential[I] := TAnalogInput.Create(
				self, CeboMsrIoIdDifferential0 + I, TStyles.Differential, I
			);
			I := I + 1;
		end;

		SetLength(_AnalogOutputs,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterAnalogOutputCount) do begin
			SetLength(_AnalogOutputs, Length(_AnalogOutputs) + 1);
			_AnalogOutputs[I] := TAnalogOutput.Create(
				self, CeboMsrIoIdAnalogOutput0 + I, I
			);
			I := I + 1;
		end;

		SetLength(_DigitalPorts,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterDigitalPortCount) do begin
			SetLength(_DigitalPorts, Length(_DigitalPorts) + 1);
			_DigitalPorts[I] := TDigitalPort.Create(
				self, CeboMsrIoIdDigitalPort0 + I, I
			);
			I := I + 1;
		end;

		SetLength(_Counters,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterCounterCount) do begin
			SetLength(_Counters, Length(_Counters) + 1);
			_Counters[I] := TCounter.Create(
				self, CeboMsrIoIdCounter0 + I, I
			);
			I := I + 1;
		end;

		SetLength(_Triggers,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterTriggerCount) do begin
			SetLength(_Triggers, Length(_Triggers) + 1);
			_Triggers[I] := TTrigger.Create(
				self, CeboMsrIoIdTrigger0 + I, I
			);
			I := I + 1;
		end;

		SetLength(_CurrentSources,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterCurrentSourceCount) do begin
			SetLength(_CurrentSources, Length(_CurrentSources) + 1);
			_CurrentSources[I] := TCurrentSource.Create(
				self, CeboMsrCurrentSource0 + I, I
			);
			I := I + 1;
		end;

		SetLength(_Leds,0);
		I := 0;
		while I < TSystemConnector.GetDeviceParameter(_Handle, CeboParameterLedCount) do begin
			SetLength(_Leds, Length(_Leds) + 1);
			_Leds[I] := TLed.Create(
				self, CeboMsrLedIndicator + I, 0
			);
			I := I + 1;
		end;

		_MaxInputFrames := TSystemConnector.GetDeviceParameter(_Handle, CeboParameterMaxInputFrames);
	end;
		
	procedure TDevice.Close();
	begin
		TSystemConnector.Close(_Handle);
	end;
	
	procedure TDevice.ResetDevice();
	begin
		TSystemConnector.ResetDevice(_Handle);
	end;
	
	procedure TDevice.ResetPeripherals(Mask: longint);
	begin
		TSystemConnector.ResetPeripherals(_Handle, Mask);
	end;
	
	function TDevice.CalculateMaxBufferedInputFrames(): longint;
	begin
		CalculateMaxBufferedInputFrames := TSystemConnector.CalculateMaxBufferedInputFrames(_Handle);
	end;
	
	procedure TDevice.SetupInputFrame( Inputs: TIInputList);
		var
			Ios : IoArray;
			Input : IInput;
	begin
		for Input in Inputs do begin
			if (Input is TAnalogInput) then begin
				SetLength(Ios, Length(Ios) + 1);
				Ios[Length(Ios) - 1] := (TAnalogInput(Input)).InterfaceId;
			end
			else if (Input is TDigitalPort) then begin
				SetLength(Ios, Length(Ios) + 1);
				Ios[Length(Ios) - 1] := (TDigitalPort(Input)).InterfaceId;
			end
			else if (Input is TTrigger) then begin
				SetLength(Ios, Length(Ios) + 1);
				Ios[Length(Ios) - 1] := (TTrigger(Input)).InterfaceId;
			end
			else if (Input is TCounter) then begin
				SetLength(Ios, Length(Ios) + 1);
				Ios[Length(Ios) - 1] := (TCounter(Input)).InterfaceId;
			end
			else begin
				raise EArgumentException.Create('Unknown input.');
			end;
		end;
		TSystemConnector.SetupInputFrame(_Handle, Ios);
	end;
			
	function TDevice.StartBufferedDataAcquisition(FrameRate: single; FrameCount: longint; ExternalStarted: boolean): single;
	begin
		if (FrameRate <= 0) then raise EArgumentException.Create('FrameRate must be > 0');
		if (FrameCount <= 0) then raise EArgumentException.Create('FrameCount must be > 0');
		StartBufferedDataAcquisition := TSystemConnector.StartDataAcquisition(_Handle, ExternalStarted, FrameRate, FrameCount);
	end;
	
	procedure TDevice.StartBufferedExternalTimedDataAcquisition(FrameCount: longint);
	begin
		if (FrameCount <= 0) then raise EArgumentException.Create('FrameCount must be > 0');
		TSystemConnector.StartDataAcquisition(_Handle, true, -1, FrameCount);
	end;
	
	function TDevice.StartContinousDataAcquisition(FrameRate: single; ExternalStarted: boolean): single;
	begin
		if (FrameRate <= 0) then raise EArgumentException.Create('FrameRate must be > 0');
		StartContinousDataAcquisition := TSystemConnector.StartDataAcquisition(_Handle, ExternalStarted, FrameRate, -1);
	end;
	
	procedure TDevice.StartContinousExternalTimedDataAcquisition();
	begin
		TSystemConnector.StartDataAcquisition(_Handle, true, -1, -1);
	end;
	
	procedure TDevice.StopDataAcquisition();
	begin
		TSystemConnector.StopDataAcquisition(_Handle);
	end;
	
	function TDevice.ReadBlocking(FrameCount: longint): TInputFrameList;
		var
			VValues: VoltageArray;
			Values: DigitalArray;
			I: longword;
			Return: TInputFrameList;
	begin
		SetLength(VValues, InputFrameFloatCount * FrameCount);
		SetLength(Values, InputFrameIntCount * FrameCount);
		TSystemConnector.ReadBlocking(_Handle, VValues, Values, FrameCount);
		SetLength(Return, FrameCount);
		I := 0;
		while I < FrameCount do begin
			Return[I] := BuildInputFrame(VValues, Values, I);
			I := I + 1;
		end;
		ReadBlocking := Return;
	end;
	
	function TDevice.ReadNonBlocking(): TInputFrameList;
		var
			VValues: VoltageArray;
			Values: DigitalArray;
			FrameCount: longint;
			I: longword;
			Return: TInputFrameList;
	begin
		SetLength(VValues, InputFrameFloatCount * _MaxInputFrames);
		SetLength(Values, InputFrameIntCount * _MaxInputFrames);
		FrameCount := TSystemConnector.ReadNonBlocking(_Handle, VValues, Values);
		SetLength(Return, FrameCount);
		I := 0;
		while I < FrameCount do begin
			Return[I] := BuildInputFrame(VValues, Values, I);
			I := I + 1;
		end;
		ReadNonBlocking := Return;
	end;
		
	function TDevice.ReadFrame(): TInputFrame;
		var
			VValues: VoltageArray;
			Values: DigitalArray;
	begin
		SetLength(VValues, InputFrameFloatCount);
		SetLength(Values, InputFrameIntCount);
		TSystemConnector.ReadFrame(_Handle, VValues, Values);
		ReadFrame := BuildInputFrame(VValues, Values, 0);
	end;
	
	procedure TDevice.SetupOutputFrame(Outputs: TIOutputList);
		var
			Ios: IoArray;
			Output: IOutput;
	begin
		for Output in Outputs do begin
			if (Output is TAnalogOutput) then begin
				SetLength(Ios, Length(Ios) + 1);
				Ios[Length(Ios) - 1] := (TAnalogOutput(Output)).InterfaceId;
			end
			else if (Output is TDigitalPort) then begin
				SetLength(Ios, Length(Ios) + 1);
				Ios[Length(Ios) - 1] := (TDigitalPort(Output)).InterfaceId;
			end
			else begin
				raise EArgumentException.Create('Unknown output.');
			end;
		end;
		TSystemConnector.SetupOutputFrame(_Handle, Ios);
	end;
		
	procedure TDevice.WriteFrame(Frame: TOutputFrame);
	begin
		TSystemConnector.WriteFrame(_Handle, Frame.VValues, Frame.Values);
	end;
	
	function TDevice.CreateOutputFrame(): TOutputFrame;
	begin
		CreateOutputFrame := TOutputFrame.Create(OutputFrameFloatCount, OutputFrameIntCount);
	end;
	
	function TDevice.ToString(): string;
	begin
		ToString := 'CeboMsr-Device: ' + string(_Ident);
	end;

//
//// TLibraryInterface
//

	class function TLibraryInterface.GetApiVersion(): ansistring;
	begin
		GetApiVersion := TSystemConnector.GetApiVersion();
	end;

	class function TLibraryInterface.Enumerate(DevType: TDeviceType): TDeviceList;
		var
			Length, I: longint;
			MsrType: longword;
			Version, Ident: ansistring;
			Return: TDeviceList;
	begin
		Length := TSystemConnector.BeginEnumerate(DevType.InterfaceId);
		SetLength(Return, Length);
		I := 0;
		while I < Length do begin
			MsrType := 0;
			Version := '';
			Ident := TSystemConnector.GetIdentifier(I, MsrType, Version);
			Return[I] := TDevice.Create(Ident, DevType.GetDeviceTypeById(MsrType), Version);
			I := I + 1;
		end;
		Enumerate := Return;
	end;
	
	class function TLibraryInterface.EnumerateNetwork(Hostname: ansistring; Port: longint; DevType: TDeviceType): TDeviceList;
		var
			Length, I: longint;
			MsrType: longword;
			Version, Ident: ansistring;
			Return: TDeviceList;
	begin
		Length := TSystemConnector.BeginEnumerateNetwork(Hostname, Port, DevType.InterfaceId);
		SetLength(Return, Length);
		I := 0;
		while I < Length do begin
			MsrType := 0;
			Version := '';
			Ident := TSystemConnector.GetIdentifier(I, MsrType, Version);
			Return[I] := TDevice.Create(Ident, DevType.GetDeviceTypeById(MsrType), Version);
			I := I + 1;
		end;
		EnumerateNetwork := Return;
	end;

initialization
	DeviceType := TDeviceTypeList.Create();
end.
