{**
	CESYS Software License - Version 1.0 - January 1st, 2016

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The software must only be used to operate hardware manufactured by 
	CESYS GmbH, Herzogenaurach.

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
  
	CeboMsr - Delphi example - Info.

	Print out various information about the device.

	@author Daniel Schroll
}

program InfoExample;
	uses
		SysUtils,
		CeboMsr in 'cebomsr\CeboMsr.pas';

	{**
		Print various information to the console.
	}
	procedure PrintInformation(Device: TDevice);
		var
			CurrentSource: TCurrentSource;
			Se0: TAnalogInput;
			I: longint;
			Range: TRange;
			Dp0: TDigitalPort;
	begin

		// Put out some device specific information.
		writeln('Device Type: ' + Device.DeviceType.Name);
		writeln('API ver.: ' + CeboMsr.TLibraryInterface.ApiVersion);
		writeln('Firmware ver.: ' + Device.FirmwareVersion);
		writeln('Identifier: ' + Device.Identifier);
		writeln('Serial number: ' + Device.SerialNumber);
		writeln('Temperature: ' + floattostr(Device.Temperature));

		// Get the real current of the reference current sources.
		for CurrentSource in Device.CurrentSources do begin
			writeln('Reference current of ' + CurrentSource.Name + ': ' +
				floattostr(CurrentSource.ReferenceCurrent) + ' uA');
		end;

		// Retrieve information about single ended input #0.
		// This works on all analog inputs and outputs.
		Se0 := Device.SingleEndedInputs[0];

		writeln('Info for single ended input ' + Se0.Name + ':');
		writeln('  Min. ICD: ' + inttostr(Se0.MinInterChannelDelay) + ' us');
		writeln('  Current range: '
			+ floattostr(Se0.Range.MinValue) + ' V to '
			+ floattostr(Se0.Range.MaxValue) + ' V');
		writeln('Supported ranges:');


		for I := 0 to (length(Se0.SupportedRanges) - 1) do begin
			Range := Se0.SupportedRanges[I];
			writeln('    Range #' + inttostr(I) + ' range: '
				+ floattostr(Range.MinValue) + ' V to '
				+ floattostr(Range.MaxValue) + ' V, '
				+ 'Def. ICD: '
				+ inttostr(Se0.GetDefaultInterChannelDelay(range)) + ' us');
		end;

		// Get info for digital port #0.
		Dp0 := Device.DigitalPorts[0];
		writeln('Info for digital port ' + Dp0.Name + ':');
		writeln('  Count of I/O''s: ' + inttostr(Dp0.IoCount));
		for I := 0 to (Dp0.IoCount - 1) do begin
			writeln('    I/O #' + inttostr(I) + ': ' + Dp0.GetIoName(I));
		end;
	end;

var
	Devices: TDeviceList;
	Device: TDevice;

begin
	try
		// Search for devices
		Devices := CeboMsr.TLibraryInterface.Enumerate(CeboMsr.DeviceType.AllDevices);

		writeln('devices.length: ' + inttostr(length(Devices)));
		writeln('device identifier: ' + Devices[0].Identifier);

		// If at least one device has been found, use the first one
		if (length(Devices) > 0) then begin
			Device := Devices[0];

			// Open device, nothing can be done without doing this
			Device.Open();

			// Put out Info
			PrintInformation(Device);

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			Device.Close();
		end else begin
			writeln('No Devices found!');
		end;
		readln;
	except
		on E: EArgumentException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIndexOutOfRangeException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EInvalidOperationException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIoException do begin
			writeln(E.Message);
			readln;
		end;
	end;
end.
