{**
	CESYS Software License - Version 1.0 - January 1st, 2016

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The software must only be used to operate hardware manufactured by 
	CESYS GmbH, Herzogenaurach.

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
  
	CeboMsr - Delphi example - Triggers.

	This example outlines how two CeboMsr devices can be chained via trigger.
	The first device is used as master, it is configured to output an alternating trigger,
	while the second device is configured to respond to this trigger.
	Both devices sample digital port #0 and single ended input #0.
	Trigger #0 is used for both master and slave.

	Requirements: Two CeboLC devices must be connected to the host, GND and TRG must be
	connected between both devices.

	This example works with CeboLC only, but can be ported to Cebo-Stick as well.

	@author Daniel Schroll
}

program TriggerExample;
	uses
		SysUtils,
		CeboMsr in 'cebomsr\CeboMsr.pas';

	{**
		Write frames to console.
		@param(Device Device string.)
		@param(Frames Frames to dump.)
	}
	procedure DumpFrames(Device: ansistring; Frames: TInputFrameList);
		var
			Frame: TInputFrame;
	begin
		for Frame in Frames do begin
			writeln(
				Device + ': ' +
				'se#0: ' + floattostr(Frame.GetSingleEnded(0)) + ' V, ' +
				'dp#0: ' + inttostr(Frame.GetDigitalPort(0))
			);
		end;
	end;

	{**
		Start DAQ, read frames and output the results.
		@param(Master Master device.)
		@param(Slave Slave device.)
	}
	procedure RunDataAcquisition(Master, Slave: TDevice);
		var
			MasterFrames, SlaveFrames: longint;
			Frames: TInputFrameList;
	begin

		// the slave must be started fist, as it reacts on the master's
		// trigger. Continous DAQ is used. Timed using an external trigger.
		Slave.StartContinousExternalTimedDataAcquisition();

		// The master uses hardware timed DAQ, continous at 50 Hz.
		// The 'false' here signals that the process should start immediately.
		Master.StartContinousDataAcquisition(50, false);

		// The example reads at least 10 frames from both devices and
		// subsequently outputs the samples.
		MasterFrames := 0;
		SlaveFrames := 0;

		while (MasterFrames < 10) or (SlaveFrames < 10) do begin
			// Start by reading frames from master,
			// output it and increment counter.
			Frames := Master.ReadNonBlocking();
			DumpFrames('master', Frames);
			MasterFrames := MasterFrames + length(Frames);

			// Do the same with the slave.
			Frames := Slave.ReadNonBlocking();
			DumpFrames('slave', Frames);
			SlaveFrames := SlaveFrames + length(Frames);

			// Don't poll too frequent, this would fully utilize one core.
			Sleep(1);
		end;

		// Finished, gracefully stop DAQ.
		Slave.StopDataAcquisition();
		Master.StopDataAcquisition();
	end;

	{**
		Both devices are fully configured here.
		@param(Master Master device.)
		@param(Slave Slave device.)
	}
	procedure Configure(Master, Slave: TDevice);
		var
			Frame: TIInputList;
	begin

		// The slave's trigger must be set to alternating as well.
		Slave.Triggers[0].Config := TTriggerConfig.InputAlternating;

		// The trigger for the master must be set to alternating output.
		Master.Triggers[0].Config := TTriggerConfig.OutputAlternating;

		// Both devices now get configured to the same input frame layout.

		// set Size of frame-items
		setLength(Frame,2);

		
		Frame[0] := Master.SingleEndedInputs[0];
		Frame[1] := Master.DigitalPorts[0];
		Master.SetupInputFrame(Frame);

		Frame[0] := Slave.SingleEndedInputs[0];
		Frame[1] := Slave.DigitalPorts[0];
		Slave.SetupInputFrame(Frame);

		writeln('Slave Trigger Config: 0x' + inttohex(integer(Slave.Triggers[0].Config),8));
		writeln('Master Trigger Config: 0x' + inttohex(integer(Master.Triggers[0].Config),8));
		writeln('output alternating: 0x' + inttohex(integer(TTriggerConfig.OutputAlternating),8));
		writeln('input alternating: 0x' + inttohex(integer(TTriggerConfig.InputAlternating),8));
	end;
		

var
	Devices: TDeviceList;
	Master, Slave: TDevice;

begin
	try
		// Search for devices
		Devices := CeboMsr.TLibraryInterface.Enumerate(CeboMsr.DeviceType.AllDevices);

		// exactly two are required.
		if (2 = length(Devices)) then begin
			// as both devices can act as master,
			// we simply use the first one for this role.
			Master := Devices[0];
			Master.Open();
			writeln('Master: ' + Master.SerialNumber + '@' + Master.Identifier);

			// and the second as slave.
			Slave := Devices[1];
			Slave.Open();
			writeln('Slave: ' + Slave.SerialNumber + '@' + Slave.Identifier);

			// Configure both
			Configure(Master, Slave);

			// and do DAQ.
			RunDataAcquisition(Master, Slave);

			// Close both.
			Master.Close();
			Slave.Close();

		end else begin
			writeln('Exactly two devices are required.');
		end;
		readln;
	except
		on E: EArgumentException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIndexOutOfRangeException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EInvalidOperationException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIoException do begin
			writeln(E.Message);
			readln;
		end;
	end;
end.
